@@ -5,10 +5,13 @@ from .dictify import (
     jsonify,
     dictify,
     normalize,
+    prepare,
     validate_all,
     ModelLookup,
     jsonify_dict,
     normalize_dict,
+    prepare_dict,
+    raise_error
 )
 from jsonschema import (
     validate,
@@ -27,22 +30,27 @@ from . import (
 class DefaultRegistry:
     jsonify = jsonify_dict
     normalize = normalize_dict
+    prepare = prepare_dict
     restriction = default_restriction_dict
     column_to_schema = default_column_to_schema
 
 
 class Mapping(object):
-    def __init__(self, validator, model, modellookup, registry=DefaultRegistry):
+    def __init__(self, validator, model, modellookup, registry=DefaultRegistry, treat_error=raise_error):
         self.validator = validator
         self.format_checker = validator.format_checker
         self.schema = validator.schema
         self.model = model
         self.modellookup = modellookup
         self.registry = registry
+        self.treat_error = treat_error
 
     def jsondict_from_object(self, ob):
         return jsonify(ob, self.schema, registry=self.registry.jsonify)
 
+    def jsondict_from_string_only_dict(self, string_only_dict):
+        return prepare(string_only_dict, self.schema, registry=self.registry.prepare)
+
     def dict_from_jsondict(self, jsondict):
         return normalize(jsondict, self.schema, registry=self.registry.normalize)
 
@@ -56,7 +64,7 @@ class Mapping(object):
         return validate(jsondict, self.schema, format_checker=self.format_checker)
 
     def validate_all_jsondict(self, jsondict):
-        return validate_all(jsondict, self.validator)
+        return validate_all(jsondict, self.validator, self.treat_error)
 
 
 class MappingFactory(object):
