@@ -100,11 +100,10 @@ def test_it_normalize__partial():
 
     factory = SchemaFactory(AlsoChildrenWalker)
     group_schema = factory(Group)
-    group_dict = {'name': 'ravenclaw', 'color': 'blue', 'pk': None}
+    group_dict = {'name': 'ravenclaw', 'color': 'blue'}
 
     result = _callFUT(group_dict, group_schema, convert=normalize_of, getter=dict.get)
-    assert result == {'pk': None, 'color': 'blue', 'name': 'ravenclaw',
-                      "created_at": None, 'users': []}
+    assert result == {'color': 'blue', 'name': 'ravenclaw', 'users': []}
 
 
 def test_it_normalize__partial2():
@@ -117,5 +116,35 @@ def test_it_normalize__partial2():
     user_dict = {'name': 'foo'}
 
     result = _callFUT(user_dict, user_schema, convert=normalize_of, getter=dict.get)
-    assert result == {'pk': None, 'name': 'foo', "created_at": None, 'group': None}
+    assert result == {'name': 'foo', 'group': None}
 
+
+def test_it__prepare():
+    from alchemyjsonschema import SchemaFactory, AlsoChildrenWalker
+    from alchemyjsonschema.tests.models import Group
+    from alchemyjsonschema.dictify import prepare_of
+
+    factory = SchemaFactory(AlsoChildrenWalker)
+    group_schema = factory(Group)
+    group_dict = {'name': 'ravenclaw', 'created_at': '2000-01-01T00:00:00+00:00', 'color': 'blue', 'pk': 1,
+                  'users': [{'name': 'foo', 'created_at': '2000-01-01T00:00:00+00:00', 'pk': 12},
+                            {'name': 'boo', 'created_at': '2000-01-01T00:00:00+00:00', 'pk': 13}]}
+
+    result = _callFUT(group_dict, group_schema, convert=prepare_of, getter=dict.get)
+    assert result == {'name': 'ravenclaw', 'created_at': '2000-01-01T00:00:00+00:00',
+                      'color': 'blue', 'pk': 1,
+                      'users': [{'name': 'foo', 'created_at': '2000-01-01T00:00:00+00:00', 'pk': 12},
+                                {'name': 'boo', 'created_at': '2000-01-01T00:00:00+00:00', 'pk': 13}]}
+
+
+def test_it__prepare_partial():
+    from alchemyjsonschema import SchemaFactory, AlsoChildrenWalker
+    from alchemyjsonschema.tests.models import Group
+    from alchemyjsonschema.dictify import prepare_of
+
+    factory = SchemaFactory(AlsoChildrenWalker)
+    group_schema = factory(Group)
+    group_dict = {'name': 'ravenclaw', 'color': 'blue', 'pk': 1}
+
+    result = _callFUT(group_dict, group_schema, convert=prepare_of, getter=dict.get)
+    assert result == {'name': 'ravenclaw', 'pk': 1, 'color': 'blue', 'users': []}
